@@ -1,4 +1,4 @@
-/* $Id: gucharmap-table.c,v 1.12 2003/06/06 22:44:07 nlevitt Exp $ */
+/* $Id: gucharmap-table.c,v 1.15 2003/08/20 18:58:56 jirka Exp $ */
 /*
  * Copyright (c) 2003  Noah Levitt <nlevitt аt columbia.edu>
  *
@@ -306,7 +306,9 @@ layout_scaled_glyph (GucharmapTable *chartable,
 
 
 static GdkPixmap *
-create_glyph_pixmap (GucharmapTable *chartable, gint font_size)
+create_glyph_pixmap (GucharmapTable *chartable, 
+                     gint font_size,
+                     gboolean draw_font_family)
 {
   enum { PADDING = 8 };
 
@@ -322,21 +324,28 @@ create_glyph_pixmap (GucharmapTable *chartable, gint font_size)
    */
   pango_layout = layout_scaled_glyph (chartable, chartable->active_char, 
                                       font_size, &family);
-
-  if (family == NULL)
-    family = _("[not a printable character]");
-
-  pango_layout2 = pango_layout_new (
-          gtk_widget_get_pango_context (GTK_WIDGET (chartable)));
-  pango_layout_set_text (pango_layout2, family, -1);
-  pango_layout_get_pixel_extents (pango_layout2, NULL, &family_rect);
-
   pango_layout_get_pixel_extents (pango_layout, &char_rect, NULL);
 
-  /* Make the GdkPixmap large enough to account for possible offsets in the
-   * ink extents of the glyph. */
-  pixmap_width  = MAX (char_rect.width, family_rect.width)  + 2 * PADDING;
-  pixmap_height = family_rect.height + char_rect.height + 4 * PADDING;
+  if (draw_font_family)
+    {
+      if (family == NULL)
+        family = g_strdup (_("[not a printable character]"));
+
+      pango_layout2 = pango_layout_new (
+              gtk_widget_get_pango_context (GTK_WIDGET (chartable)));
+      pango_layout_set_text (pango_layout2, family, -1);
+      pango_layout_get_pixel_extents (pango_layout2, NULL, &family_rect);
+
+      /* Make the GdkPixmap large enough to account for possible offsets in the
+       * ink extents of the glyph. */
+      pixmap_width  = MAX (char_rect.width, family_rect.width)  + 2 * PADDING;
+      pixmap_height = family_rect.height + char_rect.height + 4 * PADDING;
+    }
+  else
+    {
+      pixmap_width  = char_rect.width + 2 * PADDING;
+      pixmap_height = char_rect.height + 2 * PADDING;
+    }
 
   style = gtk_widget_get_style (chartable->drawing_area);
 
@@ -357,14 +366,18 @@ create_glyph_pixmap (GucharmapTable *chartable, gint font_size)
                    pango_layout);
   g_object_unref (pango_layout);
 
-  gdk_draw_line (pixmap, style->dark_gc[GTK_STATE_NORMAL],
-                 6 + 1, char_rect.height + 2 * PADDING,
-                 pixmap_width - 3 - 6, char_rect.height + 2 * PADDING);
-  gdk_draw_layout (pixmap, style->text_gc[GTK_STATE_NORMAL],
-                   PADDING, pixmap_height - PADDING - family_rect.height,
-                   pango_layout2);
+  if (draw_font_family)
+    {
+      gdk_draw_line (pixmap, style->dark_gc[GTK_STATE_NORMAL],
+                     6 + 1, char_rect.height + 2 * PADDING,
+                     pixmap_width - 3 - 6, char_rect.height + 2 * PADDING);
+      gdk_draw_layout (pixmap, style->text_gc[GTK_STATE_NORMAL],
+                       PADDING, pixmap_height - PADDING - family_rect.height,
+                       pango_layout2);
+
+      g_object_unref (pango_layout2);
+    }
 
-  g_object_unref (pango_layout2);
   g_free (family);
 
   return pixmap;
@@ -440,7 +453,7 @@ update_zoom_window (GucharmapTable *chartable)
     g_object_unref (chartable->zoom_pixmap);
 
   chartable->zoom_pixmap = create_glyph_pixmap (
-          chartable, compute_zoom_font_size (chartable));
+          chartable, compute_zoom_font_size (chartable), TRUE);
 
   if (GTK_WIDGET_REALIZED (chartable->zoom_window))
     {
@@ -1255,9 +1268,14 @@ key_release_event (GtkWidget *widget,
 {
   switch (event->keyval)
     {
+      /* XXX: If the group(shift_toggle) Xkb option is set, then releasing
+       * the shift key gives either ISO_Next_Group or ISO_Prev_Group. Is
+       * there a better way to handle this case? */
       case GDK_Shift_L: case GDK_Shift_R:
+      case GDK_ISO_Next_Group: case GDK_ISO_Prev_Group:
         gucharmap_table_zoom_disable (chartable);
         break;
+
     }
 
   return FALSE;
@@ -1720,7 +1738,8 @@ drag_begin (GtkWidget *widget,
   GdkPixmap *drag_icon;
 
   drag_icon = create_glyph_pixmap (chartable, 
-                                   compute_drag_font_size (chartable));
+                                   compute_drag_font_size (chartable),
+                                   FALSE);
   gtk_drag_set_icon_pixmap (context, gtk_widget_get_colormap (widget), 
                             drag_icon, NULL, -8, -8);
   g_object_unref (drag_icon);
